@@ -25,7 +25,7 @@ use POSIX qw(tzset);
 
 our($VERSION, $canonical_time, $clamp_time, $verbose);
 
-$VERSION = '1.6.0'; # <https://semver.org/>
+$VERSION = '1.7.0'; # <https://semver.org/>
 
 sub init() {
 	$ENV{'TZ'} = 'UTC';
@@ -64,7 +64,7 @@ sub get_normalizer_for_file($) {
 		return _handler('gzip');
 	}
 	# jar
-	if (m/\.(jar|war|hpi|apk)$/
+	if (m/\.(jar|war|hpi|apk|sym)$/
 		&& _get_file_type($_) =~ m/(Java|Zip) archive data/) {
 		return _handler('jar');
 	}
